module STEMConstants
    @distance_mode = 'meter'
    def self.set_distance_mode(mode)
        @distance_mode = mode if Distance[mode.to_sym]
    end
    
    @time_mode = 'second'
    def self.set_time_mode(mode)
        @time_mode = mode if Time[mode.to_sym]
    end
    
    @mass_mode = 'gram'
    def self.set_mass_mode(mode)
        @mass_mode = mode if Mass[mode.to_sym]
    end
    
    Number = {
        #most numbers can be found at this wiki page for explanation: https://en.wikipedia.org/wiki/Mathematical_constant#Simple_representatives_of_sets_of_numbers
        :pi => 3.141592653589793,
        :tau => 3.141592653589793 / 2,
        :golden_ratio => 1.618033988749894,
        :euler => 0.577215664901532,
        :sqrt_2 => 1.414213562373095,
        :sqrt_3 => 1.732050807568877,
        :euler_mascheroni => 0.577215664901532,
        :meissel_mertens => 0.261497212847642,
        :bernstein => 0.280169499023869,
        :sierpinski => 2.584981759579253
    }
    
    Distance = {
        :default => 1.0,
        :meter => 1.0,
        :centimeter => 100.0, 
        :millimeter => 1000.0,
        :kilometer => 1.0 / 1000.0,
        :feet => 3.280839895,
        :inch => 39.37007874,
        :yard => 1.093613298,
        :mile => 0.0006213711922
    }
    
    Time = {
        :default => 1.0,
        :second => 1.0,
        :jiffy => 100.0,
        :millisecond => 1000.0,
        :minute => 1.0 / 60.0,
        :hour => 1.0 / 60.0 / 60.0,
        :day => 1.0 / 60.0 / 60.0 / 24.0,
        :year => 1.0 / 60.0 / 60.0 / 24.0 / 365.0,
        :leap_year => 1.0 / 60.0 / 60.0 / 24.0 / 366.0,
        :corrected_year => 1.0 / 60.0 / 60.0 / 24.0 / 365.25 #the mean amount of days in a year is 365.25
    }
    
    Mass = {
        :default => 1.0,
        :gram => 1.0,
        :centigram => 100.0,
        :milligram => 1000.0,
        :kilogram => 1.0 / 1000.0,
        :ounce => 0.035274,
        :pound => 0.00220462,
        :slug => 1.0 / 1000.0 / 14.6,
        :ton => 1.0 / 1000.0 / 1000.0
    }
    
    def self.number(constant, conversion = 'default')
        Number[constant.to_sym] / Number[conversion.to_sym]
    end
    
    def self.distance(constant, conversion = 'default')
        Distance[constant.to_sym] / Distance[conversion.to_sym] / Distance[@distance_mode.to_sym]
    end
    
    def self.time(constant, conversion = 'default')
        Time[constant.to_sym] / Time[conversion.to_sym] / Time[@time_mode.to_sym]
    end
    
    def self.mass(constant, conversion = 'default')
        Mass[c.to_sym] / Mass[conversion.to_sym] / Mass[@mass_mode.to_sym]
    end
end